use crate::{extra_info::ExtraInfo, get_date};
use lox_library::bridge_table::BridgeLine;
use std::collections::BTreeMap;

// The Bridge struct only tracks data for today
pub struct Bridge {
    pub fingerprint: [u8; 20],
    real_connections: u32,
    total_connections: u32,
}

impl Bridge {
    pub fn new(fingerprint: &[u8; 20]) -> Self {
        Self {
            fingerprint: *fingerprint,
            real_connections: 0,
            total_connections: 0,
        }
    }

    pub fn from_bridge_line(bridgeline: &BridgeLine) -> Self {
        Self::new(&bridgeline.get_hashed_fingerprint())
    }

    pub fn connect_real(&mut self) {
        self.real_connections += 1;
        self.total_connections += 1;
    }

    pub fn connect_total(&mut self) {
        self.total_connections += 1;
    }

    // Let the censor simulate a bunch of connections at once
    pub fn censor_flood(&mut self, num_connections: u32) {
        self.total_connections += num_connections;
    }

    // Generate an extra-info report for today
    pub fn gen_extra_info(&self, country: &str) -> ExtraInfo {
        let mut bridge_ips = BTreeMap::<String, u32>::new();
        // Round up to a multiple of 8
        let rounded_connection_count =
            self.total_connections + 7 - (self.total_connections + 7) % 8;
        //let rounded_connection_count = (self.total_connections + 7) / 8 * 8;
        bridge_ips.insert(country.to_string(), rounded_connection_count);
        ExtraInfo {
            nickname: String::from("simulation-bridge"),
            fingerprint: self.fingerprint,
            date: get_date(),
            bridge_ips,
        }
    }

    pub fn reset_for_tomorrow(&mut self) {
        self.real_connections = 0;
        self.total_connections = 0;
    }
}
